@@ -30,6 +30,7 @@ void init_keybindings()
   global_bindings.set("Remove", '-');
   global_bindings.set("Hold", '=');
   global_bindings.set("Purge", '_');
+  global_bindings.set("Keep", ':');
   global_bindings.set("SetAuto", 'M');
   global_bindings.set("ClearAuto", 'm');
   global_bindings.set("ForbidUpgrade", 'F');
@@ -55,12 +56,15 @@ void init_keybindings()
   global_bindings.set("DescriptionCycle", "i"[0]);
 
   global_bindings.set("DpkgReconfigure", "R"[0]);
+  global_bindings.set("BugReport", "B"[0]);
 
   // Hierarchy editor
   global_bindings.set("Commit", "N"[0]);
   global_bindings.set("SaveHier", "S"[0]);
   global_bindings.set("EditHier", "E"[0]);
 
+  global_bindings.set("SearchBroken", "b"[0]);
+
   pkg_tree::init_bindings();
   pkg_tree_node::init_bindings();
   cmine::init_bindings();
@@ -82,10 +86,6 @@ void init_colors()
   set_color("DownloadHit", COLOR_BLACK, COLOR_GREEN, 0);
 
   set_color("Bullet", COLOR_YELLOW, COLOR_BLACK, A_BOLD);
-
-  // For displaying package info
-  set_color("FromDep", COLOR_BLUE, COLOR_BLACK, A_BOLD);
-  set_color("ToDep", COLOR_GREEN, COLOR_BLACK, A_BOLD);
 }
 
 void init_defaults()
