@@ -20,6 +20,7 @@
 #include <vscreen/config/keybindings.h>
 
 #include <sigc++/bind.h>
+#include <sigc++/object_slot.h>
 
 #include <algorithm>
 
@@ -29,7 +30,7 @@ using namespace std;
 static void download_summary_nasty_hack(SigC::Slot0<void> cancel_slot)
 {
   if(cancel_slot)
-    cancel_slot.call();
+    cancel_slot();
 
   vscreen_exitmain();
 }
@@ -51,35 +52,35 @@ static vscreen_widget *download_summary(download_list *l,
     rval=vs_dialog_ok(buf, SigC::slot(vscreen_exitmain));
   else
     rval=vs_dialog_yesno(buf,
-			 SigC::slot(vscreen_exitmain), "Continue",
+			 SigC::slot(vscreen_exitmain), _("Continue"),
 			 SigC::bind(SigC::slot(download_summary_nasty_hack),
-				    cancel_slot), "Cancel",
+				    cancel_slot), _("Cancel"),
 			 get_color("DefaultWidgetBackground")|A_REVERSE);
 
   rval->connect_key("PrevPage",
 		    &global_bindings,
-		    slot(l, &download_list::pageup));
+		    slot(*l, &download_list::pageup));
   rval->connect_key("Up",
 		    &global_bindings,
-		    slot(l, &download_list::lineup));
+		    slot(*l, &download_list::lineup));
   rval->connect_key("NextPage",
 		    &global_bindings,
-		    slot(l, &download_list::pagedown));
+		    slot(*l, &download_list::pagedown));
   rval->connect_key("Down",
 		    &global_bindings,
-		    slot(l, &download_list::linedown));
+		    slot(*l, &download_list::linedown));
   rval->connect_key("Begin",
 		    &global_bindings,
-		    slot(l, &download_list::skip_to_top));
+		    slot(*l, &download_list::skip_to_top));
   rval->connect_key("End",
 		    &global_bindings,
-		    slot(l, &download_list::skip_to_bottom));
+		    slot(*l, &download_list::skip_to_bottom));
   rval->connect_key("Left",
 		    &global_bindings,
-		    slot(l, &download_list::shift_left));
+		    slot(*l, &download_list::shift_left));
   rval->connect_key("Right",
 		    &global_bindings,
-		    slot(l, &download_list::shift_right));
+		    slot(*l, &download_list::shift_right));
 
   rval->show_all();
 
@@ -96,7 +97,7 @@ download_list::download_list(Slot0Arg _abortslot, bool _display_messages)
    TotalItems(0), CurrentItems(0),
    CurrentCPS(0), TotalBytes(0), CurrentBytes(0)
 {
-  do_layout.connect(slot(this, &download_list::layout_me));
+  do_layout.connect(slot(*this, &download_list::layout_me));
 }
 
 // FIXME: just discarding this is not only bad -- it's also dangerous: it's a
@@ -136,7 +137,7 @@ void download_list::paint()
       int width1;
 
       if(workers[where].total!=0)
-	width1=(width*workers[where].current)/workers[where].total;
+	width1=(int) ((((double) width)*(double) workers[where].current)/(double) workers[where].total);
       else
 	width1=0;
 
@@ -265,7 +266,7 @@ void download_list::IMSHit(pkgAcquire::ItemDesc &itmdesc,
 {
   if(display_messages)
     {
-      msgs.push_back(pair<string, int>(itmdesc.Description+" [Hit]",
+      msgs.push_back(pair<string, int>(itmdesc.Description+" "+_("[Hit]"),
 					    get_color("DownloadHit")));
 
       sync_top();
@@ -285,8 +286,8 @@ void download_list::Done(pkgAcquire::ItemDesc &itmdesc,
 {
   if(display_messages)
     {
-      msgs.push_back(pair<string, int>(itmdesc.Description+" [Downloaded]",
-					    get_color("DownloadProgress")));
+      msgs.push_back(pair<string, int>(itmdesc.Description+" "+_("[Downloaded]"),
+				       get_color("DownloadProgress")));
 
       sync_top();
 
@@ -305,11 +306,11 @@ void download_list::Fail(pkgAcquire::ItemDesc &itmdesc,
 
       // ???
       if(itmdesc.Owner->Status==pkgAcquire::Item::StatDone)
-	msgs.push_back(pair<string, int>(itmdesc.Description+" [IGNORED]",
+	msgs.push_back(pair<string, int>(itmdesc.Description+" "+_("[IGNORED]"),
 					      get_color("DownloadHit")));
       else
 	{
-	  msgs.push_back(pair<string, int>(itmdesc.Description+" [ERROR]",
+	  msgs.push_back(pair<string, int>(itmdesc.Description+" "+_("[ERROR]"),
 						get_color("Error")));
 	  msgs.push_back(pair<string, int>(" "+itmdesc.Owner->ErrorText,
 						get_color("Error")));
