@@ -21,25 +21,32 @@
 
 #include "apt.h"
 #include "../aptitude.h"
-#include "undo.h"
+
+
 #include "config_signal.h"
 #include "pkg_hier.h"
+#include "rev_dep_iterator.h"
 #include "tasks.h"
+#include "undo.h"
 
-#include <apt-pkg/sourcelist.h>
-#include <apt-pkg/pkgcachegen.h>
-#include <apt-pkg/error.h>
 #include <apt-pkg/configuration.h>
-#include <apt-pkg/init.h>
 #include <apt-pkg/depcache.h>
+#include <apt-pkg/error.h>
+#include <apt-pkg/init.h>
+#include <apt-pkg/pkgcachegen.h>
+#include <apt-pkg/sourcelist.h>
+#include <apt-pkg/version.h>
+
+#include <fstream>
 
 #include <assert.h>
-#include <stdio.h>
 #include <signal.h>
 #include <unistd.h>
 #include <sys/stat.h>
 #include <errno.h>
 
+using namespace std;
+
 aptitudeCacheFile *apt_cache_file=NULL;
 signalling_config *aptcfg=NULL;
 pkgRecords *apt_package_records=NULL;
@@ -49,7 +56,8 @@ pkg_hier *user_pkg_hier=NULL;
 string *pendingerr=NULL;
 bool erroriswarning=false;
 
-static Configuration theme_config;
+static Configuration *theme_config;
+static Configuration *user_config;
 
 SigC::Signal0<void> cache_closed, cache_reloaded, cache_reload_failed;
 SigC::Signal0<void> hier_reloaded;
@@ -76,7 +84,10 @@ void apt_preinit()
 {
   signal(SIGPIPE, SIG_IGN);
 
-  ReadConfigFile(theme_config, PKGDATADIR "/aptitude-defaults");
+  theme_config=new Configuration;
+  user_config=new Configuration;
+
+  ReadConfigFile(*theme_config, PKGDATADIR "/aptitude-defaults");
 
   pkgInitConfig(*_config);
   pkgInitSystem(*_config, _system);
@@ -91,15 +102,19 @@ void apt_preinit()
 
       // This attempts to test whether the file is available
       if(access(cfgloc.c_str(), R_OK)==0)
-	ReadConfigFile(*_config, cfgloc);
+	{
+	  ReadConfigFile(*user_config, cfgloc);
+	  ReadConfigFile(*_config, cfgloc);
+	}
     }
 
-  aptcfg=new signalling_config(_config, &theme_config);
+  aptcfg=new signalling_config(user_config, _config, theme_config);
+
+  apt_dumpcfg("Aptitude");
 
   apt_undos=new undo_list;
 }
 
-// Hacked from the APT sources
 void apt_dumpcfg(const char *root)
 {
   const char *HOME=getenv("HOME");
@@ -118,7 +133,7 @@ void apt_dumpcfg(const char *root)
 
   cfgloc+="/config";
 
-  FILE *f=fopen((cfgloc+".new").c_str(), "w");
+  ofstream f((cfgloc+".new").c_str());
 
   if(!f)
     {
@@ -126,33 +141,9 @@ void apt_dumpcfg(const char *root)
       return;
     }
 
-  /* Write out all of the configuration directives by walking the 
-     configuration tree */
-  const Configuration::Item *Top = _config->Tree(root), *Curr=Top;
-  if(Curr)
-    do
-      {
-	// Exclude theme configuration from being saved.
-	// (note: this is just a specific instance of a general problem,
-	// that we save configuration items that the user hasn't messed with)
-	if(strcasecmp(Curr->FullTag().c_str(), PACKAGE "::Themes"))
-	  {
-	    fprintf(f, "%s \"%s\";\n", Curr->FullTag().c_str(), Curr->Value.c_str());
-      
-	    if (Curr->Child != 0)
-	      {
-		Curr = Curr->Child;
-		continue;
-	      }
-	  }
-      
-	while (Curr != 0 && Curr->Next == 0 && Curr->Parent!=Top)
-	  Curr = Curr->Parent;
-	if (Curr != 0)
-	  Curr = Curr->Next;
-      } while(Curr && Curr!=Top);
+  aptcfg->Dump(f);
 
-  fclose(f);
+  f.close();
 
   if(rename((cfgloc+".new").c_str(), cfgloc.c_str())!=0)
     {
@@ -164,13 +155,21 @@ void apt_dumpcfg(const char *root)
 // Revert back to the default set of options.  Is it a hack?  Hmmm...
 void apt_revertoptions()
 {
+  Configuration *old_user_config=user_config;
+  Configuration *old_config=_config;
+
   _config=new Configuration;
 
+  user_config=new Configuration;
+
   // ick?
   pkgInitConfig(*_config);
   pkgInitSystem(*_config, _system);
 
-  delete aptcfg->setcfg(_config);
+  aptcfg->setcfg(user_config, _config, theme_config);
+
+  delete old_user_config;
+  delete old_config;
 }
 
 void apt_init(OpProgress *progress_bar, bool do_initselections,
@@ -358,3 +357,99 @@ void surrounding_or(pkgCache::DepIterator dep,
   // If not, something is wrong with apt's cache.
   assert(found);
 }
+
+bool package_suggested(const pkgCache::PkgIterator &pkg)
+{
+  pkgDepCache::StateCache &state=(*apt_cache_file)[pkg];
+  pkgCache::VerIterator candver=state.CandidateVerIter(*apt_cache_file);
+
+  for(rev_dep_iterator d(pkg); !d.end(); ++d)
+    if((*d)->Type==pkgCache::Dep::Suggests)
+      {
+	bool satisfied=false;
+
+	pkgCache::DepIterator start,end;
+
+	surrounding_or(*d, start, end);
+
+	while(start!=end)
+	  {
+	    if(((*apt_cache_file)[start])&pkgDepCache::DepGInstall)
+	      {
+		satisfied=true;
+		break;
+	      }
+
+	    ++start;
+	  }
+
+	if(!satisfied)
+	  {
+	    // Check whether the package doing the depending is going
+	    // to be installed.
+	    pkgCache::PkgIterator depender=(*d).ParentPkg();
+	    pkgDepCache::StateCache &depstate=(*apt_cache_file)[depender];
+	    pkgCache::VerIterator depinstver=depstate.InstVerIter(*apt_cache_file);
+
+	    if(depender.CurrentVer().end() &&
+	       depstate.Install() &&
+	       !depinstver.end() &&
+	       !candver.end() &&
+	       _system->VS->CheckDep(candver.VerStr(),
+				     (*d)->CompareOp, (*d).TargetVer()))
+	      {
+		if((*d).ParentVer()==depinstver)
+		  return true;
+	      }
+	  }
+      }
+
+  return false;
+}
+
+bool package_recommended(const pkgCache::PkgIterator &pkg)
+{
+  pkgDepCache::StateCache &state=(*apt_cache_file)[pkg];
+  pkgCache::VerIterator candver=state.CandidateVerIter(*apt_cache_file);
+
+  for(rev_dep_iterator d(pkg); !d.end(); ++d)
+    if((*d)->Type==pkgCache::Dep::Recommends)
+      {
+	bool satisfied=false;
+
+	pkgCache::DepIterator start,end;
+
+	surrounding_or(*d, start, end);
+
+	while(start!=end)
+	  {
+	    if(((*apt_cache_file)[start])&pkgDepCache::DepGInstall)
+	      {
+		satisfied=true;
+		break;
+	      }
+
+	    ++start;
+	  }
+
+	if(!satisfied)
+	  {
+	    // Check whether the package doing the depending is going
+	    // to be installed or upgraded.
+	    pkgCache::PkgIterator depender=(*d).ParentPkg();
+	    pkgDepCache::StateCache &depstate=(*apt_cache_file)[depender];
+	    pkgCache::VerIterator depinstver=depstate.InstVerIter(*apt_cache_file);
+
+	    if(depstate.Install() &&
+	       !candver.end() &&
+	       _system->VS->CheckDep(candver.VerStr(),
+				     (*d)->CompareOp, (*d).TargetVer()))
+	      {
+		if((*d).ParentVer()==depinstver)
+		  return true;
+	      }
+	  }
+      }
+
+  return false;
+}
