@@ -130,4 +130,15 @@ void surrounding_or(pkgCache::DepIterator dep,
 		    pkgCache::DepIterator &start,
 		    pkgCache::DepIterator &end);
 
+
+/** \return true if pkg is suggested by another package which will be
+ *  installed.
+ */
+bool package_suggested(const pkgCache::PkgIterator &pkg);
+
+/** \return true if pkg is recommended by another package which will
+ *  be installed or upgraded.
+ */
+bool package_recommended(const pkgCache::PkgIterator &pkg);
+
 #endif
