@@ -193,6 +193,9 @@ private:
   // The current 'group level' -- how many times start_action_group has been
   // called without a matching end_action_group.
 
+  /** The number of "new" packages. */
+  int new_package_count;
+
   bool mark_and_sweep_in_progress;
   // Tracks whether a mark-and-sweep is running to avoid infinite recursion.
   // FIXME: disgusting hack.
@@ -218,7 +221,9 @@ private:
 
   // gc
   void mark_package(const PkgIterator &pkg,
-		    const VerIterator &ver);
+		    const VerIterator &ver,
+		    bool follow_recommends,
+		    bool follow_suggests);
   void mark_and_sweep(undo_group *undo);
 
   // The following three methods exist so that undos function sensibly.
@@ -251,6 +256,15 @@ public:
   // Clears all information about which packages are 'new'.  Overwrites undoer
   // if it is not NULL
 
+  /** Sets the "new" flag for the given package; use this instead of
+   *  directly manipulating the cache so the count of new packages is
+   *  updated. (gross?)
+   */
+  void set_new_flag(const pkgCache::PkgIterator &pkg, bool is_new);
+
+  /** Gets the number of new packages. */
+  int get_new_package_count() const {return new_package_count;}
+
   inline aptitude_state &get_ext_state(const PkgIterator &Pkg)
   {return package_states[Pkg->ID];}
 
@@ -307,7 +321,7 @@ public:
    */
   bool is_held(const PkgIterator &pkg);
 
-  bool all_upgrade(undo_group *undo);
+  bool all_upgrade(bool with_autoinst, undo_group *undo);
   // Wrapper for pkgAllUpgrade (the engine of "apt-get upgrade")
 
   bool try_fix_broken(undo_group *undo);
