@@ -27,15 +27,16 @@
 #include <generic/config_signal.h>
 #include <generic/tasks.h>
 
-#include <apt-pkg/strutl.h>
 #include <apt-pkg/error.h>
-#include <apt-pkg/version.h>
 #include <apt-pkg/pkgsystem.h>
+#include <apt-pkg/policy.h>
+#include <apt-pkg/strutl.h>
+#include <apt-pkg/version.h>
 
 #include <unistd.h>
 
 column_definition_list *pkg_item::pkg_columnizer::columns=NULL;
-const char *pkg_item::pkg_columnizer::default_pkgdisplay="%c%a%M %p #%v%V";
+const char *pkg_item::pkg_columnizer::default_pkgdisplay="%c%a%M %p %Z %v %V";
 
 // NOTE: the default widths here will be overridden by the initialization
 //      code.
@@ -58,11 +59,13 @@ column_type_defaults pkg_item::pkg_columnizer::defaults[pkg_columnizer::numtypes
   {1, false, false},    // autoset
   {1, false, false},    // tagged
   {10, true, true},     // archive
+  {7, false, false},    // sizechange
   {strlen(PACKAGE), false, false},  // progname
   {strlen(VERSION), false, false},  // progver
   {12, false, true},    // brokencount
   {30, false, true},    // diskusage
   {15, false, true},    // downloadsize
+  {4, false, false},    // pin_priority
   {15, false, true}
 };
 
@@ -135,6 +138,25 @@ column_disposition pkg_item::pkg_columnizer::setup_column(int type)
       else
 	return column_disposition("", 0);
       break;
+    case sizechange:
+      if(pkg.end())
+	return column_disposition("", 0);
+      else
+	{
+	  pkgCache::VerIterator inst_ver=(*apt_cache_file)[pkg].InstVerIter(*apt_cache_file);
+
+	  // TODO: move this to a generic file (common with cmdline.cc)
+	  int dsize=(inst_ver.end()?0:inst_ver->InstalledSize)
+	    -(pkg.CurrentVer().end()?0:pkg.CurrentVer()->InstalledSize);
+
+	  if(dsize==0)
+	    return column_disposition("", 0);
+	  else if(dsize>0)
+	    return column_disposition("+"+SizeToStr(dsize)+"B", 0);
+	  else
+	    return column_disposition("-"+SizeToStr(dsize)+"B", 0);
+	}
+      break;
     case currver:
       if(pkg.end())
 	return column_disposition("", 0);
@@ -391,6 +413,30 @@ column_disposition pkg_item::pkg_columnizer::setup_column(int type)
       }
 
       break;
+    case pin_priority:
+      {
+	if(apt_cache_file && !pkg.end())
+	  {
+	    char buf[256];
+
+	    pkgPolicy *policy=dynamic_cast<pkgPolicy *>(&(*apt_cache_file)->GetPolicy());
+
+	    if(!policy)
+	      return column_disposition("", 0);
+
+	    // Not quite sure what this indicates
+	    signed short priority=policy->GetPriority(pkg);
+
+	    if(priority==0)
+	      return column_disposition("", 0);
+
+	    snprintf(buf, 256, "%d", priority);
+
+	    return column_disposition(buf, 0);
+	  }
+	else
+	  return column_disposition("", 0);
+      }
     case autoset:
       // Display the "auto" flag UNLESS the package has been removed
       // or purged already and is not presently being installed.
@@ -451,13 +497,15 @@ column_disposition pkg_item::pkg_columnizer::setup_column(int type)
 	     _system->VS->CheckDep(visible_ver.VerStr(), D->CompareOp, D.TargetVer()))
 	    count++;
 
-	for(pkgCache::PrvIterator i=visible_ver.ProvidesList(); !i.end(); i++)
+	for(pkgCache::PrvIterator i=pkg.ProvidesList(); !i.end(); i++)
 	  for(pkgCache::DepIterator D=i.ParentPkg().RevDependsList(); !D.end(); D++)
-	    if(D.IsCritical() &&
-	       D->Type!=pkgCache::Dep::Conflicts &&
-	       D.ParentVer()==D.ParentPkg().CurrentVer() &&
-	       _system->VS->CheckDep(i.ProvideVersion(), D->CompareOp, D.TargetVer()))
-	      count++;
+	    {
+	      if(D.IsCritical() &&
+		 D->Type!=pkgCache::Dep::Conflicts &&
+		 D.ParentVer()==D.ParentPkg().CurrentVer() &&
+		 _system->VS->CheckDep(i.ProvideVersion(), D->CompareOp, D.TargetVer()))
+		count++;
+	    }
 	snprintf(buf, 100, "%i", count);
 	return column_disposition(buf, 0);
       }
@@ -521,6 +569,10 @@ int pkg_item::pkg_columnizer::parse_column_type(char id)
       return downloadsize;
     case 'H':
       return hostname;
+    case 'Z':
+      return sizechange;
+    case 'i':
+      return pin_priority;
     default:
       return -1;
     }
