@@ -53,9 +53,10 @@ public:
   enum types {name, installed_size, debsize, stateflag, actionflag,
 	      description, currver, candver, longstate, longaction,
 	      maintainer, priority, shortpriority, section, revdepcount,
-	      autoset, tagged, archive,
+	      autoset, tagged, archive, sizechange,
 
               progname, progver, brokencount, diskusage, downloadsize,
+	      pin_priority,
               hostname, numtypes};
   static column_type_defaults defaults[numtypes];
   static const char *column_names[numtypes];
