@@ -29,16 +29,23 @@
 #include "pkg_ver_item.h"
 #include "ui.h"
 #include "view_changelog.h"
+#include "vs_progress.h"
 
 #include "generic/apt.h"
+#include "generic/apt_undo_group.h"
 #include "generic/config_signal.h"
 
 #include "vscreen/vs_multiplex.h"
 
+#include <algorithm>
 #include <string>
+
 #include <apt-pkg/configuration.h>
+#include <apt-pkg/policy.h>
 #include <apt-pkg/strutl.h>
 
+using namespace std;
+
 class pkg_ver_columnizer:public pkg_item::pkg_columnizer
 {
   bool show_pkg_name;
@@ -95,6 +102,62 @@ column_disposition pkg_ver_columnizer::setup_column(int type)
 	return column_disposition(_("<N/A>"), 0);
 
       break;
+
+    case pin_priority:
+      {
+	if(apt_cache_file && !ver.end())
+	  {
+	    char buf[256];
+
+	    pkgPolicy *policy=dynamic_cast<pkgPolicy *>(&(*apt_cache_file)->GetPolicy());
+
+	    if(!policy)
+	      return column_disposition("", 0);
+
+	    pkgCache::VerFileIterator vf=ver.FileList();
+
+	    if(vf.end())
+	      return column_disposition("", 0);
+
+	    signed short priority=policy->GetPriority(vf.File());
+
+	    ++vf;
+
+	    // Find the highest priority for this version.
+	    while(!vf.end())
+	      {
+		priority=max(priority, policy->GetPriority(vf.File()));
+		++vf;
+	      }
+
+	    snprintf(buf, 256, "%d", priority);
+
+	    return column_disposition(buf, 0);
+	  }
+	else
+	  return column_disposition("", 0);
+      }
+
+    case sizechange:
+      if(ver.end())
+	return column_disposition("", 0);
+
+      {
+	pkgCache::PkgIterator pkg=ver.ParentPkg();
+
+	pkgCache::VerIterator curver=pkg.CurrentVer();
+	pkgCache::VerIterator instver=(*apt_cache_file)[pkg].InstVerIter(*apt_cache_file);
+
+	if((ver==curver && ver==instver) ||
+	   (ver!=curver && ver!=instver))
+	  return column_disposition("", 0);
+	else if(ver==curver)
+	  return column_disposition("-"+SizeToStr(ver->InstalledSize)+"B", 0);
+	else
+	  return column_disposition("+"+SizeToStr(ver->InstalledSize)+"B", 0);
+      }
+
+      break;
     case debsize:
       if(ver.end())
 	return column_disposition("", 0);
@@ -479,7 +542,7 @@ void pkg_ver_item::select(undo_group *undo)
     }
   else
     {
-      undo_group *grp=undo?new undo_group:NULL;
+      undo_group *grp=undo?new apt_undo_group:NULL;
       (*apt_cache_file)->set_candidate_version(version, grp);
       (*apt_cache_file)->mark_install(version.ParentPkg(), _config->FindB(PACKAGE "::Auto-Install", true), false, grp);
 
@@ -513,6 +576,11 @@ void pkg_ver_item::hold(undo_group *undo)
 				 undo);
 }
 
+void pkg_ver_item::keep(undo_group *undo)
+{
+  // Do nothing for now.
+}
+
 void pkg_ver_item::purge(undo_group *undo)
 {
   if(version==version.ParentPkg().CurrentVer())
@@ -533,6 +601,18 @@ void pkg_ver_item::set_auto(bool isauto, undo_group *undo)
   (*apt_cache_file)->mark_auto_installed(version.ParentPkg(), isauto, undo);
 }
 
+void pkg_ver_item::forbid_version(undo_group *undo)
+{
+  if(version!=version.ParentPkg().CurrentVer())
+    (*apt_cache_file)->forbid_upgrade(version.ParentPkg(), version.VerStr(), undo);
+}
+
+void pkg_ver_item::show_information()
+{
+  vscreen_widget *w=make_info_screen(version.ParentPkg(), version);
+  insert_main_widget(w, string(_("Information about "))+version.ParentPkg().Name(), "");
+}
+
 pkg_ver_screen::pkg_ver_screen(const pkgCache::PkgIterator &pkg)
   :apt_info_tree(pkg.Name(), "")
 {
@@ -572,13 +652,43 @@ bool pkg_ver_item::dispatch_char(chtype ch, vs_tree *owner)
     }
   else if(bindings->key_matches(ch, "InfoScreen"))
     {
-      vscreen_widget *w=make_info_screen(version.ParentPkg(), version);
-      insert_main_widget(w, string(_("Information about "))+version.ParentPkg().Name(), "");
+      show_information();
       return true;
     }
   else if(bindings->key_matches(ch, "Changelog"))
     {
       view_changelog(version);
+      return true;
+    }
+  else if(bindings->key_matches(ch, "ForbidUpgrade"))
+    {
+      undo_group *grp=new apt_undo_group;
+      forbid_version(grp);
+
+      if(!grp->empty())
+	apt_undos->add_item(grp);
+      else
+	delete grp;
+
+      return true;
+    }
+  else if(bindings->key_matches(ch, "BugReport"))
+    {
+      // Try to report a bug on the package.  (ew quoting ew)
+      string cmd=string("reportbug '")+version.ParentPkg().Name()+"' -V '"+version.VerStr()+"'";
+
+      vscreen_suspend();
+
+      apt_cache_file->ReleaseLock();
+
+      printf(_("Reporting a bug in %s:\n"), version.ParentPkg().Name());
+
+      system(cmd.c_str());
+
+      vscreen_resume();
+
+      apt_reload_cache(gen_progress_bar(), true);
+
       return true;
     }
   else
