@@ -8,6 +8,7 @@
 
 #include "desc_parse.h"
 #include "edit_pkg_hier.h"
+#include "menu_redirect.h"
 #include "pkg_columnizer.h"
 #include "reason_fragment.h"
 #include "ui.h"
@@ -101,9 +102,13 @@ public:
   }
 };
 
-static void do_set_column_format(string s, pkg_handling_label *l)
+static void do_set_column_format(string key, string the_default,
+				 pkg_handling_label *l)
 {
-  column_definition_list *columns=parse_columns(s, pkg_item::pkg_columnizer::parse_column_type,
+  string format=aptcfg->Find(key, the_default.c_str());
+
+  column_definition_list *columns=parse_columns(format,
+						pkg_item::pkg_columnizer::parse_column_type,
 						pkg_item::pkg_columnizer::defaults);
 
   if(!columns)
@@ -278,8 +283,27 @@ public:
   }
 };
 
+static bool do_menu_test(vscreen_widget *view,
+			 menu_redirect *redirect,
+			 bool (menu_redirect::* test)())
+{
+  return active_main_widget()==view &&
+    (redirect->*test)();
+}
+
+static bool do_menu_action(vscreen_widget *view,
+			   menu_redirect *redirect,
+			   bool (menu_redirect::* action)())
+{
+  if(active_main_widget()==view)
+    return (redirect->*action)();
+  else
+    return false;
+}
+
 vscreen_widget *make_package_view(list<package_view_item> &format,
 				  vscreen_widget *mainwidget,
+				  menu_redirect *menu_handler,
 				  pkg_signal *sig, desc_signal *desc_sig,
 				  bool show_reason_first)
 {
@@ -314,8 +338,10 @@ vscreen_widget *make_package_view(list<package_view_item> &format,
 
 	      if(!i->columns_cfg.empty())
 		aptcfg->connect(i->columns_cfg,
-				SigC::bind(SigC::slot(do_set_column_format),
-					   l));
+				SigC::bind(bind(SigC::slot(do_set_column_format),
+						l),
+					   i->columns_cfg,
+					   i->columns_cfg_default));
 	    }
 	  break;
 	case PACKAGE_VIEW_DESCRIPTION:
@@ -437,5 +463,131 @@ vscreen_widget *make_package_view(list<package_view_item> &format,
   if(!mainwidget)
     _error->Error(_("make_package_view: no main widget found"));
 
+  if(menu_handler)
+    {
+      // Connect the appropriate signals..
+
+      // TODO: need to change the condition for enabling the menu to
+      // check whether the overall widget is visible...
+      package_menu_enabled.connect(bind(bind(SigC::slot(do_menu_test),
+					     &menu_redirect::package_enabled),
+					rval,
+					menu_handler));
+
+      package_install.connect(bind(bind(SigC::slot(do_menu_action),
+					&menu_redirect::package_install),
+				   rval,
+				   menu_handler));
+
+      package_remove.connect(bind(bind(SigC::slot(do_menu_action),
+				       &menu_redirect::package_remove),
+				  rval,
+				  menu_handler));
+
+      package_purge.connect(bind(bind(SigC::slot(do_menu_action),
+				      &menu_redirect::package_purge),
+				 rval,
+				 menu_handler));
+
+      package_hold.connect(bind(bind(SigC::slot(do_menu_action),
+				     &menu_redirect::package_hold),
+				rval,
+				menu_handler));
+
+      package_keep.connect(bind(bind(SigC::slot(do_menu_action),
+				     &menu_redirect::package_keep),
+				rval,
+				menu_handler));
+
+      package_mark_auto.connect(bind(bind(SigC::slot(do_menu_action),
+				     &menu_redirect::package_mark_auto),
+				rval,
+				menu_handler));
+
+      package_unmark_auto.connect(bind(bind(SigC::slot(do_menu_action),
+					    &menu_redirect::package_unmark_auto),
+				       rval,
+				       menu_handler));
+
+      package_forbid_enabled.connect(bind(bind(SigC::slot(do_menu_test),
+					       &menu_redirect::package_forbid_enabled),
+					  rval,
+					  menu_handler));
+
+      package_forbid.connect(bind(bind(SigC::slot(do_menu_action),
+				     &menu_redirect::package_forbid),
+				rval,
+				menu_handler));
+
+      package_information_enabled.connect(bind(bind(SigC::slot(do_menu_test),
+						    &menu_redirect::package_information_enabled),
+					       rval,
+					       menu_handler));
+
+      package_information.connect(bind(bind(SigC::slot(do_menu_action),
+					    &menu_redirect::package_information),
+				       rval,
+				       menu_handler));
+
+      package_changelog_enabled.connect(bind(bind(SigC::slot(do_menu_test),
+						  &menu_redirect::package_changelog_enabled),
+					     rval,
+					     menu_handler));
+
+      package_changelog.connect(bind(bind(SigC::slot(do_menu_action),
+					  &menu_redirect::package_changelog),
+				     rval, menu_handler));
+
+      find_search_enabled.connect(bind(bind(SigC::slot(do_menu_test),
+					    &menu_redirect::find_search_enabled),
+				       rval,
+				       menu_handler));
+
+      find_search.connect(bind(bind(SigC::slot(do_menu_action),
+				    &menu_redirect::find_search),
+			       rval,
+			       menu_handler));
+
+      find_research_enabled.connect(bind(bind(SigC::slot(do_menu_test),
+					      &menu_redirect::find_research_enabled),
+					 rval,
+					 menu_handler));
+
+      find_research.connect(bind(bind(SigC::slot(do_menu_action),
+				      &menu_redirect::find_research),
+				 rval,
+				 menu_handler));
+
+      find_limit_enabled.connect(bind(bind(SigC::slot(do_menu_test),
+					   &menu_redirect::find_limit_enabled),
+				      rval,
+				      menu_handler));
+
+      find_limit.connect(bind(bind(SigC::slot(do_menu_action),
+				   &menu_redirect::find_limit),
+			      rval,
+			      menu_handler));
+
+      find_cancel_limit_enabled.connect(bind(bind(SigC::slot(do_menu_test),
+						  &menu_redirect::find_reset_limit_enabled),
+					     rval,
+					     menu_handler));
+
+      find_cancel_limit.connect(bind(bind(SigC::slot(do_menu_action),
+					  &menu_redirect::find_reset_limit),
+				     rval,
+				     menu_handler));
+
+      find_broken_enabled.connect(bind(bind(SigC::slot(do_menu_test),
+					    &menu_redirect::find_broken_enabled),
+				       rval,
+				       menu_handler));
+
+      find_broken.connect(bind(bind(SigC::slot(do_menu_action),
+				    &menu_redirect::find_broken),
+			       rval,
+			       menu_handler));
+    }
+
   return rval;
 }
