@@ -52,6 +52,7 @@
 #include <mine/cmine.h>
 
 #include <generic/apt.h>
+#include <generic/apt_undo_group.h>
 #include <generic/config_signal.h>
 
 #include "dep_item.h"
@@ -99,13 +100,41 @@ vscreen_widget *active_status_download;
 SigC::Signal0<void> file_quit;
 SigC::Signal0<void> package_states_changed;
 
+SigC::Signal0<bool, MarshalOr> package_menu_enabled;
+SigC::Signal0<bool, MarshalOr> package_forbid_enabled;
+SigC::Signal0<bool, MarshalOr> package_information_enabled;
+SigC::Signal0<bool, MarshalOr> package_changelog_enabled;
+
+SigC::Signal0<bool, MarshalOr> find_search_enabled;
+SigC::Signal0<bool, MarshalOr> find_research_enabled;
+SigC::Signal0<bool, MarshalOr> find_limit_enabled;
+SigC::Signal0<bool, MarshalOr> find_cancel_limit_enabled;
+SigC::Signal0<bool, MarshalOr> find_broken_enabled;
+
+SigC::Signal0<bool, MarshalOr> package_install;
+SigC::Signal0<bool, MarshalOr> package_remove;
+SigC::Signal0<bool, MarshalOr> package_purge;
+SigC::Signal0<bool, MarshalOr> package_hold;
+SigC::Signal0<bool, MarshalOr> package_keep;
+SigC::Signal0<bool, MarshalOr> package_mark_auto;
+SigC::Signal0<bool, MarshalOr> package_unmark_auto;
+SigC::Signal0<bool, MarshalOr> package_forbid;
+SigC::Signal0<bool, MarshalOr> package_information;
+SigC::Signal0<bool, MarshalOr> package_changelog;
+
+SigC::Signal0<bool, MarshalOr> find_search;
+SigC::Signal0<bool, MarshalOr> find_research;
+SigC::Signal0<bool, MarshalOr> find_limit;
+SigC::Signal0<bool, MarshalOr> find_cancel_limit;
+SigC::Signal0<bool, MarshalOr> find_broken;
+
 const char *default_pkgstatusdisplay="%d";
 const char *default_pkgheaderdisplay="%N %n #%B %u %o";
 const char *default_grpstr="filter(missing),task,status,section(subdir,passthrough),section(topdir)";
 
 class apt_error_pager:public vs_text_layout
 {
-  unsigned int maxlen;
+  string::size_type maxlen;
 public:
   apt_error_pager():vs_text_layout(), maxlen(0)
   {
@@ -124,11 +153,11 @@ public:
 
 	maxlen=max(maxlen, tag.size()+1+currerr.size());
 
-	frags.push_back(fragf("%s %s", tag.c_str(), currerr.c_str()));
+	frags.push_back(indentbox(0, 3,
+				  wrapbox(fragf("%s %s", tag.c_str(), currerr.c_str()))));
       }
 
-    set_fragment(attr_fragment(indentbox(0, 3,
-					 wrapbox(sequence_fragment(frags))),
+    set_fragment(attr_fragment(sequence_fragment(frags),
 			       get_color("Error")));
   }
 
@@ -143,7 +172,7 @@ public:
     // direction once X has been finalized).  However, this takes a lot of
     // effort and testing and is not suitable for the current aptitude
     // release (which hopefully will be what goes into sarge)
-    unsigned int top_w=rootwin.getmaxx();
+    string::size_type top_w=rootwin.getmaxx();
 
     // Allow 2 characters for the frame and 1 for the scrollbar.
     if(top_w<3)
@@ -331,6 +360,18 @@ static void do_su_to_root(string args)
     }
 }
 
+static bool su_to_root_enabled()
+{
+  return getuid()!=0;
+}
+
+static void update_menubar_autohide()
+{
+  main_menu->set_always_visible(main_multiplex->num_children()==0 ||
+				!aptcfg->FindB(PACKAGE "::UI::Menubar-Autohide",
+					       false));
+}
+
 vscreen_widget *reload_message;
 static void do_show_reload_message()
 {
@@ -381,11 +422,20 @@ static void do_destroy_visible()
 
       // If all the screens are destroyed, we make the menu visible (so the
       // user knows that something is still alive :) )
-      if(main_multiplex->num_children()==0)
-	main_menu->set_always_visible(true);
+      update_menubar_autohide();
     }
 }
 
+static bool view_next_prev_enabled()
+{
+  return main_multiplex->num_visible()>1;
+}
+
+static bool any_view_visible()
+{
+  return main_multiplex->visible_widget()!=NULL;
+}
+
 // These are necessary because main_multiplex isn't created until after
 // the slot in the static initializer..
 // (creating the menu info at a later point would solve this problem)
@@ -420,15 +470,21 @@ static void do_show_dependency_options_dlg()
   w->show();
 }
 
+static void really_do_revert_options()
+{
+  apt_revertoptions();
+  apt_dumpcfg(PACKAGE);
+}
+
 static void do_revert_options()
 {
   prompt_yesno(_("Really discard your personal settings and reload the defaults?"),
 	       false,
-	       SigC::slot(apt_revertoptions),
+	       SigC::slot(really_do_revert_options),
 	       NULL);
 }
 
-static vscreen_widget *make_default_view(vscreen_widget *mainwidget,
+static vscreen_widget *make_default_view(pkg_menu_tree *mainwidget,
 					 pkg_signal *sig,
 					 desc_signal *desc_sig,
 					 bool allow_visible_desc=true,
@@ -440,7 +496,8 @@ static vscreen_widget *make_default_view(vscreen_widget *mainwidget,
 
       if(format)
 	{
-	  vscreen_widget *rval=make_package_view(*format, mainwidget, sig,
+	  vscreen_widget *rval=make_package_view(*format, mainwidget,
+						 mainwidget, sig,
 						 desc_sig, show_reason_first);
 	  delete format;
 
@@ -490,7 +547,7 @@ static vscreen_widget *make_default_view(vscreen_widget *mainwidget,
 					   "ShowHideDescription", "",
 					   allow_visible_desc && aptcfg->FindB(PACKAGE "::UI::Description-Visible-By-Default", true)));
 
-  return make_package_view(basic_format, mainwidget, sig, desc_sig,
+  return make_package_view(basic_format, mainwidget, mainwidget, sig, desc_sig,
 			   show_reason_first);
 }
 
@@ -543,7 +600,7 @@ void do_new_hier_view(OpProgress &progress)
 
   pkg_tree *tree=new pkg_tree(grpstr.c_str(), grp, NULL, progress);
   tree->set_limit("!~v");
-  tree->set_hierarchical(false);
+  //tree->set_hierarchical(false);
 
   add_main_widget(make_default_view(tree,
 				    &tree->selected_signal,
@@ -613,6 +670,10 @@ static void do_help_help()
 
   snprintf(buf, 512, HELPDIR "/%s", _("help.txt"));
 
+  // Deal with missing localized docs.
+  if(access(buf, R_OK)!=0)
+    strncpy(buf, HELPDIR "/help.txt", 512);
+
   vscreen_widget *w=vs_dialog_fileview(buf, NULL, slot(pager_search));
   w->show_all();
 
@@ -625,6 +686,10 @@ static void do_help_readme()
 
   snprintf(buf, 512, HELPDIR "/%s", _("README")); // README can be translated..
 
+  // Deal with missing localized docs.
+  if(access(buf, R_OK)!=0)
+    strncpy(buf, HELPDIR "/README", 512);
+
   vscreen_widget *w=vs_dialog_fileview(buf, NULL, slot(pager_search));
   w->show_all();
   popup_widget(w);
@@ -724,7 +789,7 @@ static void do_show_preview()
 				       &global_bindings,
 				       SigC::slot(actually_do_package_run));
       add_main_widget(active_preview, _("Preview of package installation"),
-		      "");
+		      _("View and/or adjust the actions that will be performed"));
     }
   else
     active_preview_tree->show();
@@ -749,7 +814,7 @@ static void actually_do_package_run()
 	    {
 	      if(_config->FindB(PACKAGE "::Auto-Fix-Broken", true))
 		{
-		  undo_group *undo=new undo_group;
+		  undo_group *undo=new apt_undo_group;
 		  (*apt_cache_file)->try_fix_broken(undo);
 		  if(!undo->empty())
 		    apt_undos->add_item(undo);
@@ -827,7 +892,8 @@ void do_package_run_or_show_preview()
       show_message(_("No packages are scheduled to be installed, removed, or upgraded."));
       return;
     }
-  else if(!some_non_simple_keep_happening)
+  else if(!some_non_simple_keep_happening &&
+	  !aptcfg->FindB(PACKAGE "::Allow-Null-Upgrade", false))
     {
       show_message(_("No packages will be installed, removed or upgraded.\n\n"
 		     "Some packages could be upgraded, but you have not chosen to upgrade them.\n"
@@ -858,7 +924,7 @@ static void fix_pkgs_and_do_package_run()
 {
   if(apt_cache_file)
     {
-      undo_group *undo=new undo_group;
+      undo_group *undo=new apt_undo_group;
       (*apt_cache_file)->try_fix_broken(undo);
       if(!undo->empty())
 	apt_undos->add_item(undo);
@@ -869,6 +935,11 @@ static void fix_pkgs_and_do_package_run()
     }
 }
 
+static bool can_start_download()
+{
+  return !active_download;
+}
+
 void do_package_run()
 {
   if(apt_cache_file)
@@ -999,7 +1070,7 @@ static void do_mark_upgradable()
 {
   if(apt_cache_file)
     {
-      undo_group *undo=new undo_group;
+      undo_group *undo=new apt_undo_group;
 
       (*apt_cache_file)->mark_all_upgradable(true, undo);
 
@@ -1012,6 +1083,14 @@ static void do_mark_upgradable()
     }
 }
 
+static bool forget_new_enabled()
+{
+  if(!apt_cache_file)
+    return false;
+  else
+    return (*apt_cache_file)->get_new_package_count()>0;
+}
+
 void do_forget_new()
 {
   if(apt_cache_file)
@@ -1025,6 +1104,11 @@ void do_forget_new()
     }
 }
 
+static bool undo_enabled()
+{
+  return apt_undos->size()>0;
+}
+
 static void do_undo()
 {
   apt_undos->undo();
@@ -1061,15 +1145,15 @@ vs_menu_info actions_menu[]={
   //SigC::bind(SigC::slot(&silly_test_error), "This is a test error item.")},
 
   vs_menu_info(vs_menu_info::VS_MENU_ITEM, N_("^Install/remove packages"), "DoInstallRun",
-	       N_("Perform all pending installs and removals"), SigC::slot(do_package_run)),
+	       N_("Perform all pending installs and removals"), SigC::slot(do_package_run), SigC::slot(can_start_download)),
 
   vs_menu_info(vs_menu_info::VS_MENU_ITEM, N_("^Update package list"), "UpdatePackageList",
-	       N_("Check for new versions of packages"), SigC::slot(do_update_lists)),
+	       N_("Check for new versions of packages"), SigC::slot(do_update_lists), SigC::slot(can_start_download)),
 
   // FIXME: this is a bad name for the menu item.
   vs_menu_info(vs_menu_info::VS_MENU_ITEM, N_("^Forget new packages"), "ForgetNewPackages",
 	       N_("Forget which packages are \"new\""),
-	       SigC::slot(do_forget_new)),
+	       SigC::slot(do_forget_new), SigC::slot(forget_new_enabled)),
 
   vs_menu_info(vs_menu_info::VS_MENU_ITEM, N_("^Clean package cache"), NULL,
 	       N_("Delete package files which were previously downloaded"),
@@ -1087,11 +1171,11 @@ vs_menu_info actions_menu[]={
 	       N_("Waste time trying to find mines"), SigC::slot(do_sweep)),
 
   vs_menu_info(vs_menu_info::VS_MENU_ITEM, N_("^Become root"), NULL,
-	       N_("Run 'su' to become root.  This will restart the program, but your\nsettings will be preserved."), bind(SigC::slot(do_su_to_root), "")),
+	       N_("Run 'su' to become root; this will restart the program, but your\nsettings will be preserved"), bind(SigC::slot(do_su_to_root), ""), SigC::slot(su_to_root_enabled)),
 
 #ifdef WITH_RELOAD_CACHE
   vs_menu_info(vs_menu_info::VS_MENU_ITEM, N_("^Reload package cache"), NULL,
-	       N_("Reload the package cache."),
+	       N_("Reload the package cache"),
 	       SigC::slot(do_reload_cache)),
 #endif
 
@@ -1105,11 +1189,78 @@ vs_menu_info actions_menu[]={
 
 vs_menu_info undo_menu[]={
   vs_menu_info(vs_menu_info::VS_MENU_ITEM, N_("Undo"), "Undo",
-	       N_("Undo the last package operation or group of operations"), SigC::slot(do_undo)),
+	       N_("Undo the last package operation or group of operations"),
+	       SigC::slot(do_undo), SigC::slot(undo_enabled)),
 
   VS_MENU_END
 };
 
+vs_menu_info package_menu[]={
+  vs_menu_info(vs_menu_info::VS_MENU_ITEM, N_("^Install"), "Install",
+	       N_("Flag the currently selected package for installation or upgrade"),
+	       package_install.slot(),
+	       package_menu_enabled.slot()),
+  vs_menu_info(vs_menu_info::VS_MENU_ITEM, N_("^Remove"), "Remove",
+	       N_("Flag the currently selected package for removal"),
+	       package_remove.slot(),
+	       package_menu_enabled.slot()),
+  vs_menu_info(vs_menu_info::VS_MENU_ITEM, N_("^Purge"), "Purge",
+	       N_("Flag the currently selected package and its configuration files for removal"),
+	       package_purge.slot(),
+	       package_menu_enabled.slot()),
+  vs_menu_info(vs_menu_info::VS_MENU_ITEM, N_("^Keep"), "Keep",
+	       N_("Cancel any action on the selected package"),
+	       package_keep.slot(),
+	       package_menu_enabled.slot()),
+  vs_menu_info(vs_menu_info::VS_MENU_ITEM, N_("^Hold"), "Hold",
+	       N_("Cancel any action on the selected package, and protect it from future upgrades"),
+	       package_hold.slot(),
+	       package_menu_enabled.slot()),
+  vs_menu_info(vs_menu_info::VS_MENU_ITEM, N_("Mark ^Auto"), "SetAuto",
+	       N_("Mark the selected package as having been automatically installed;\nit will automatically be removed if no other packages depend on it"),
+	       package_mark_auto.slot(),
+	       package_menu_enabled.slot()),
+  vs_menu_info(vs_menu_info::VS_MENU_ITEM, N_("Mark ^Manual"), "ClearAuto",
+	       N_("Mark the selected package as having been manually installed;\nit will not be removed unless you manually remove it"),
+	       package_unmark_auto.slot(),
+	       package_menu_enabled.slot()),
+  vs_menu_info(vs_menu_info::VS_MENU_ITEM, N_("^Forbid Version"), "ForbidUpgrade",
+	       N_("Forbid the candidate version of the selected package from being installed;\nnewer versions of the package will be installed as usual"),
+	       package_forbid.slot(),
+	       package_forbid_enabled.slot()),
+  VS_MENU_SEPARATOR,
+  vs_menu_info(vs_menu_info::VS_MENU_ITEM, N_("I^nformation"), "InfoScreen",
+	       N_("Display more information about the selected package"),
+	       package_information.slot(),
+	       package_information_enabled.slot()),
+  vs_menu_info(vs_menu_info::VS_MENU_ITEM, N_("^Changelog"), "Changelog",
+	       N_("Display the Debian changelog of the selected package"),
+	       package_changelog.slot(),
+	       package_changelog_enabled.slot()),
+  VS_MENU_END
+};
+
+vs_menu_info search_menu[]={
+  vs_menu_info(vs_menu_info::VS_MENU_ITEM, N_("^Find"), "Search",
+	       N_("Search for a package"),
+	       find_search.slot(), find_search_enabled.slot()),
+  vs_menu_info(vs_menu_info::VS_MENU_ITEM, N_("Find ^Again"), "ReSearch",
+	       N_("Repeat the last search"),
+	       find_research.slot(), find_research_enabled.slot()),
+  VS_MENU_SEPARATOR,
+  vs_menu_info(vs_menu_info::VS_MENU_ITEM, N_("^Limit Display"),
+	       "ChangePkgTreeLimit", N_("Apply a filter to the package list"),
+	       find_limit.slot(), find_limit_enabled.slot()),
+  vs_menu_info(vs_menu_info::VS_MENU_ITEM, N_("^Un-Limit Display"),
+	       NULL, N_("Remove the filter from the package list"),
+	       find_cancel_limit.slot(), find_cancel_limit_enabled.slot()),
+  VS_MENU_SEPARATOR,
+  vs_menu_info(vs_menu_info::VS_MENU_ITEM, N_("Find ^Broken"),
+	       "SearchBroken", N_("Find the next package with unsatisfied dependencies"),
+	       find_broken.slot(), find_broken_enabled.slot()),
+  VS_MENU_END
+};
+
 vs_menu_info options_menu[]={
   vs_menu_info(vs_menu_info::VS_MENU_ITEM, N_("^UI options"), NULL,
 	       N_("Change the settings which affect the user interface"),
@@ -1138,13 +1289,16 @@ vs_menu_info options_menu[]={
 
 vs_menu_info views_menu_info[]={
   vs_menu_info(vs_menu_info::VS_MENU_ITEM, N_("^Next"), "CycleNext",
-	       N_("View next display"), SigC::slot(do_view_next)),
+	       N_("View next display"), SigC::slot(do_view_next),
+	       SigC::slot(view_next_prev_enabled)),
 
   vs_menu_info(vs_menu_info::VS_MENU_ITEM, N_("^Prev"), "CyclePrev",
-	       N_("View previous display"), SigC::slot(do_view_prev)),
+	       N_("View previous display"), SigC::slot(do_view_prev),
+	       SigC::slot(view_next_prev_enabled)),
 
   vs_menu_info(vs_menu_info::VS_MENU_ITEM, N_("^Close"), "Quit",
-	       N_("Close this display"), SigC::slot(do_destroy_visible)),
+	       N_("Close this display"), SigC::slot(do_destroy_visible),
+	       SigC::slot(any_view_visible)),
 
   VS_MENU_SEPARATOR,
 
@@ -1202,12 +1356,6 @@ static void munge_menu(vs_menu_info *menu)
     }
 }
 
-static void do_set_menubar_autohide(bool autohide)
-{
-  main_menu->set_always_visible(main_multiplex->num_children()==0 ||
-				!autohide);
-}
-
 static void do_show_menu_description(vs_menu_item *item, vs_label *label)
 {
   if(item && item->get_description().size()>0)
@@ -1223,7 +1371,7 @@ static void do_show_menu_description(vs_menu_item *item, vs_label *label)
 }
 
 // I want discardable signal arguments!
-static void do_setup_columns(std::string s)
+static void do_setup_columns()
 {
   pkg_item::pkg_columnizer::setup_columns(true);
 }
@@ -1232,17 +1380,49 @@ static void load_options(string base, bool usetheme)
 {
   load_colors(base+"::Colors", usetheme);
   load_bindings(base+"::Keybindings", &global_bindings, usetheme);
-  load_bindings(base+"::Keybindings::Minesweeper", cmine::bindings, usetheme);
   load_bindings(base+"::Keybindings::EditLine", vs_editline::bindings, usetheme);
+  load_bindings(base+"::Keybindings::Menu", vs_menu::bindings, usetheme);
   load_bindings(base+"::Keybindings::Menubar", vs_menubar::bindings, usetheme);
+  load_bindings(base+"::Keybindings::Minesweeper", cmine::bindings, usetheme);
   load_bindings(base+"::Keybindings::MinibufChoice", vs_statuschoice::bindings, usetheme);
   load_bindings(base+"::Keybindings::Pager", vs_pager::bindings, usetheme);
+  load_bindings(base+"::KeyBindings::PkgNode", pkg_tree_node::bindings, usetheme);
+  load_bindings(base+"::Keybindings::PkgTree", pkg_tree::bindings, usetheme);
+  load_bindings(base+"::Keybindings::Table", vs_table::bindings, usetheme);
+  load_bindings(base+"::Keybindings::TextLayout", vs_text_layout::bindings, usetheme);
   load_bindings(base+"::Keybindings::Tree", vs_tree::bindings, usetheme);
-  //load_bindings(base+"Keybindings::AptTree", apt_tree::bindings);
-  //load_bindings(base+"Keybindings::PkgTree", pkg_tree::bindings);
-  //load_bindings(base+"Keybindings::PkgNode", pkg_tree_node::bindings);
 }
 
+static vs_menu *add_menu(vs_menu_info *info, const std::string &name,
+			 vs_label *menu_description)
+{
+  munge_menu(info);
+
+  vs_menu *menu=new vs_menu(0, 0, 0, info);
+
+  main_menu->append_item(name, menu);
+
+  menu->item_highlighted.connect(bind(slot(do_show_menu_description),
+				      menu_description));
+
+  return menu;
+}
+
+// argh
+class help_bar:public vs_label
+{
+public:
+  help_bar(string txt, int attr):vs_label(txt, attr)
+  {
+    set_visibility();
+  }
+
+  inline void set_visibility()
+  {
+    set_visible(aptcfg->FindB(PACKAGE "::UI::HelpBar", true));
+  }
+};
+
 void ui_init()
 {
   vscreen_init();
@@ -1262,8 +1442,8 @@ void ui_init()
 
   main_menu=new vs_menubar(!aptcfg->FindB(PACKAGE "::UI::Menubar-Autohide", false));
 
-  aptcfg->connectB(string(PACKAGE "::UI::Menubar-Autohide"),
-		   SigC::slot(do_set_menubar_autohide));
+  aptcfg->connect(string(PACKAGE "::UI::Menubar-Autohide"),
+		  SigC::slot(update_menubar_autohide));
   aptcfg->connect(string(PACKAGE "::UI::Package-Display-Format"),
 		  SigC::slot(do_setup_columns));
 
@@ -1271,35 +1451,13 @@ void ui_init()
   cache_reloaded.connect(SigC::slot(do_hide_reload_message));
   cache_reload_failed.connect(SigC::slot(do_hide_reload_message));
 
-  munge_menu(actions_menu);
-  vs_menu *menu=new vs_menu(0, 0, 15, actions_menu);
-  main_menu->append_item(_("Actions"), menu);
-  menu->item_highlighted.connect(bind(slot(do_show_menu_description),
-				      menu_description));
-
-  munge_menu(undo_menu);
-  menu=new vs_menu(0, 0, 15, undo_menu);
-  main_menu->append_item(_("Undo"), menu);
-  menu->item_highlighted.connect(bind(slot(do_show_menu_description),
-				      menu_description));
-
-  munge_menu(options_menu);
-  menu=new vs_menu(0, 0, 15, options_menu);
-  main_menu->append_item(_("Options"), menu);
-  menu->item_highlighted.connect(bind(slot(do_show_menu_description),
-				      menu_description));
-
-  munge_menu(views_menu_info);
-  views_menu=new vs_menu(0, 0, 15, views_menu_info);
-  main_menu->append_item(_("Views"), views_menu);
-  views_menu->item_highlighted.connect(bind(slot(do_show_menu_description),
-					    menu_description));
-
-  munge_menu(help_menu_info);
-  menu=new vs_menu(0, 0, 15, help_menu_info);
-  main_menu->append_item(_("Help"), menu);
-  menu->item_highlighted.connect(bind(slot(do_show_menu_description),
-				      menu_description));
+  add_menu(actions_menu, _("Actions"), menu_description);
+  add_menu(undo_menu, _("Undo"), menu_description);
+  add_menu(package_menu, _("Package"), menu_description);
+  add_menu(search_menu, _("Search"), menu_description);
+  add_menu(options_menu, _("Options"), menu_description);
+  views_menu=add_menu(views_menu_info, _("Views"), menu_description);
+  add_menu(help_menu_info, _("Help"), menu_description);
 
   main_stacked=new vs_stacked;
   main_menu->set_subwidget(main_stacked);
@@ -1351,12 +1509,12 @@ void ui_init()
 	   update_key.c_str(),
 	   install_key.c_str());
 
-  vscreen_widget *help_label=new vs_label(buf, get_color("ScreenStatusColor"));
+  help_bar *help_label=new help_bar(buf, get_color("ScreenStatusColor"));
   main_table->add_widget_opts(help_label, 0, 0, 1, 1,
 			      vs_table::EXPAND | vs_table::SHRINK,
 			      vs_table::ALIGN_CENTER);
-  aptcfg->connectB(string(PACKAGE "::UI::HelpBar"),
-		   SigC::slot(*help_label, &vscreen_widget::set_visible));
+  aptcfg->connect(string(PACKAGE "::UI::HelpBar"),
+		  SigC::slot(*help_label, &help_bar::set_visibility));
 
   main_multiplex=new vs_multiplex;
   main_table->add_widget_opts(main_multiplex, 1, 0, 1, 1,
@@ -1379,10 +1537,9 @@ void ui_init()
   check_apt_errors();
   main_hook.connect(SigC::slot(check_apt_errors));
 
-  if(aptcfg->FindB(PACKAGE "::UI::HelpBar", true))
-    help_label->show();
-  else
-    help_label->hide();
+  help_label->set_visibility();
+
+  update_menubar_autohide();
 
   // Force parsing of the column stuff.
   // FIXME: put this in load_options() and kill all other references
@@ -1447,28 +1604,27 @@ static void main_widget_destroyed()
 void add_main_widget(vscreen_widget *w, const std::string &menuref,
 		     const std::string &menudesc)
 {
-  // Restore the autohide setting if necessary.
-  if(aptcfg->FindB(PACKAGE "::UI::Menubar-Autohide") &&
-     main_multiplex->num_children()==0)
-    main_menu->set_always_visible(false);
-
   setup_main_widget(w, menuref, menudesc);
   main_multiplex->add_widget(w);
   w->show();
   w->destroyed.connect(SigC::slot(main_widget_destroyed));
+
+  update_menubar_autohide();
 }
 
 void insert_main_widget(vscreen_widget *w, const std::string &menuref,
 			const std::string &menudesc)
 {
-  // Restore the autohide setting if necessary.
-  if(aptcfg->FindB(PACKAGE "::UI::Menubar-Autohide") &&
-     main_multiplex->num_children()==0)
-    main_menu->set_always_visible(false);
-
   setup_main_widget(w, menuref, menudesc);
   main_multiplex->add_widget_after(w, main_multiplex->visible_widget());
   w->show();
+
+  update_menubar_autohide();
+}
+
+vscreen_widget *active_main_widget()
+{
+  return main_multiplex->visible_widget();
 }
 
 vs_progress *gen_progress_bar()
@@ -1495,6 +1651,7 @@ static void reset_status_download()
 
 download_manager *gen_download_progress(bool force_noninvasive,
 					const string &title,
+					const string &longtitle,
 					Slot0Arg abortslot)
 {
   download_manager *m=new download_manager;
@@ -1511,7 +1668,7 @@ download_manager *gen_download_progress(bool force_noninvasive,
   else
     {
       w=new download_list(abortslot);
-      add_main_widget(w, title, "");
+      add_main_widget(w, title, longtitle);
     }
 
   m->MediaChange_sig.connect(slot(*w, &download_list::MediaChange));
@@ -1605,7 +1762,13 @@ void prompt_yesno(const std::string &prompt,
 {
   if(aptcfg->FindB(PACKAGE "::UI::Minibuf-Prompts"))
     {
-      vs_statuschoice *c=new vs_statuschoice(prompt, deflt?"yn":"ny");
+      string yesstring, nostring;
+
+      yesstring+=_("yes_key")[0];
+      nostring+=_("no_key")[0];
+      string yesnostring=deflt?yesstring+nostring:nostring+yesstring;
+
+      vs_statuschoice *c=new vs_statuschoice(prompt, yesnostring.c_str());
       c->set_bg(get_color("ScreenStatusColor"));
       c->chosen.connect(bind(bind(SigC::slot(&do_prompt_yesno),
 				  yesslot,
