@@ -68,7 +68,7 @@ vscreen_widget *vs_menubar::get_focus()
 bool vs_menubar::get_cursorvisible()
 {
   vscreen_widget *w=get_focus();
-  return (w && w->get_cursorvisible()) || active;
+  return (w && w->get_cursorvisible()) || (!w && active);
 }
 
 point vs_menubar::get_cursorloc()
@@ -194,7 +194,15 @@ void vs_menubar::layout_me()
 
       size req=(*i)->size_request();
       if(getmaxx()<menux+req.w)
-	req.w=getmaxx()-menux;
+	{
+	  if(getmaxx()>=req.w)
+	    menux=getmaxx()-req.w;
+	  else
+	    {
+	      menux=0;
+	      req.w=getmaxx();
+	    }
+	}
 
       if(getmaxy()<1+req.h)
 	req.h=getmaxy()-1;
@@ -503,7 +511,7 @@ void vs_menubar::dispatch_mouse(short id, int x, int y, int z, mmask_t bmask)
 
 void vs_menubar::init_bindings()
 {
-  bindings=new keybindings(global_bindings);
+  bindings=new keybindings(&global_bindings);
 }
 
 void vs_menubar::set_always_visible(bool _always_visible)
