@@ -143,9 +143,7 @@ vscreen_widget *vs_dialog_fileview(string fn,
 				   int attr)
 {
   vs_file_pager *p=new vs_file_pager(fn);
-  vs_scrollbar *scrollbar=new vs_scrollbar(vs_scrollbar::VERTICAL,
-					   p->get_cury(),
-					   p->get_maxy());
+  vs_scrollbar *scrollbar=new vs_scrollbar(vs_scrollbar::VERTICAL, 0, 0);
   vs_table *t=new vs_table;
   vs_frame *f=new vs_frame(t);
   vs_center *c=new vs_center(f);
@@ -172,7 +170,8 @@ vscreen_widget *vs_dialog_fileview(string fn,
 		     vs_table::ALIGN_CENTER);
 
   t->connect_key("Confirm", &global_bindings, b->pressed.slot());
-  p->location_changed.connect(slot(*scrollbar, &vs_scrollbar::set_slider));
+  p->line_changed.connect(slot(*scrollbar, &vs_scrollbar::set_slider));
+  p->do_line_signal();
   scrollbar->scrollbar_interaction.connect(slot(*p, &vs_pager::scroll_page));
 
   if(searchslot)
